package Irpg::Users;

use strict;
use warnings;
use Irpg::Utils qw(:data :text);
use Irpg::Irc qw(:interaction @queue);
use Irpg::Action;
use Irpg::Main;

my $opts;
my $rps;
my $onchan;
my ($primnick_ref, $lastreg_ref, $silentmode_ref, $pausemode_ref);
=head1 FUNCTION init_pkg
    This function sets the references to
    options and players hashes.
=over
=item SCALAR (ref)    - reference to the options hash
=item SCALAR (ref)    - reference to the players hash
=cut
sub init_pkg { 
    ($opts, $rps, $onchan, $primnick_ref,
     $lastreg_ref, $silentmode_ref, $pausemode_ref) = @_;
}

sub register {
    my ($userhost, $usernick, $username, $source, @arg) = @_;

    if (defined $username) {
        Irpg::Irc::privmsg("Sorry, you are already online as $username.",
                $usernick);
    }
    else {
        if ($#arg < 2 || $arg[2] eq "") {
            Irpg::Irc::privmsg("Try: REGISTER <char name> <password> <title>",
                    $usernick);
            Irpg::Irc::privmsg("IE : REGISTER Poseidon MyPassword M God of the ".
                    "Sea",$usernick);
        }
        elsif ($$pausemode_ref) {
            Irpg::Irc::privmsg("Sorry, new accounts may not be registered ".
                    "while the bot is in pause mode; please wait ".
                    "a few minutes and try again.",$usernick);
        }
        elsif (exists $rps->{$arg[0]} || ($opts->{casematters} &&
               scalar(grep { lc($arg[0]) eq lc($_) } keys(%$rps)))) {
            Irpg::Irc::privmsg("Sorry, that character name is already in use.",
                    $usernick);
        }
        elsif (lc($arg[0]) eq lc($opts->{botnick}) ||
               lc($arg[0]) eq lc($$primnick_ref)) {
            Irpg::Irc::privmsg("Sorry, that character name cannot be ".
                    "registered.",$usernick);
        }
        elsif (!exists($onchan->{$usernick})) {
            Irpg::Irc::privmsg("Sorry, you're not in $opts->{botchan}.",
                    $usernick);
        }
        elsif (length($arg[0]) > 16 || length($arg[0]) < 1) {
            Irpg::Irc::privmsg("Sorry, character names must be < 17 and > 0 ".
                    "chars long.", $usernick);
        }
        elsif ($arg[0] =~ /^#/) {
            Irpg::Irc::privmsg("Sorry, character names may not begin with #.",
                    $usernick);
        }
        elsif ($arg[0] =~ /\001/) {
            Irpg::Irc::privmsg("Sorry, character names may not include ".
                    "character \\001.",$usernick);
        }
        elsif ($opts->{noccodes} && ($arg[0] =~ /[[:cntrl:]]/ ||
               "@arg[2..$#arg]" =~ /[[:cntrl:]]/)) {
            Irpg::Irc::privmsg("Sorry, neither character names nor titles".
                    "may include control codes.",$usernick);
        }
        elsif ($opts->{nononp} && ($arg[0] =~ /[[:^print:]]/ ||
               "@arg[2..$#arg]" =~ /[[:^print:]]/)) {
            Irpg::Irc::privmsg("Sorry, neither character names nor titles ".
                    "may include non-printable chars.",$usernick);
        }
        elsif (length("@arg[3..$#arg]") > 30) {
            Irpg::Irc::privmsg("Sorry, character titles must be < 31 chars ".
                    "long.",$usernick);
        }
        elsif (time() == $$lastreg_ref) {
            Irpg::Irc::privmsg("Wait 1 second and try again.",$usernick);                
        }
        else {
            if ($opts->{voiceonlogin}) {
            Irpg::Irc::sts("MODE $opts->{botchan} +v :$usernick");
            }
            $$lastreg_ref = time();
            $rps->{$arg[0]}{next} = $opts->{rpbase};
            $rps->{$arg[0]}{title} = "@arg[2..$#arg]";
            $rps->{$arg[0]}{level} = 0;
            foreach (qw(str con wis int cha dex)) {
                $rps->{$arg[0]}{stats}{$_} = 1;
            }
            $rps->{$arg[0]}{class} = Irpg::Classes::Farmer->new($rps->{$arg[0]}{stats});
            $rps->{$arg[0]}{points} = 0;
            $rps->{$arg[0]}{actions} = 0;
            $rps->{$arg[0]}{next_f} = 0;
            $rps->{$arg[0]}{online} = 1;
            $rps->{$arg[0]}{nick} = $usernick;
            $rps->{$arg[0]}{userhost} = $userhost;
            $rps->{$arg[0]}{created} = time();
            $rps->{$arg[0]}{lastlogin} = time();
            $rps->{$arg[0]}{pass} = crypt($arg[1],mksalt());
            $rps->{$arg[0]}{x} = int(rand($opts->{mapx}));
            $rps->{$arg[0]}{y} = int(rand($opts->{mapy}));
            $rps->{$arg[0]}{alignment}="Neutral";
            $rps->{$arg[0]}{gender}="n";
            $rps->{$arg[0]}{isadmin} = 0;
            for my $item ("ring","amulet","charm","weapon","helm",
                          "tunic","pair of gloves","shield",
                          "set of leggings","pair of boots") {
                $rps->{$arg[0]}{item}{$item} = 0;
            }
            for my $pen ("pen_mesg","pen_nick","pen_part",
                         "pen_kick","pen_quit","pen_quest",
                         "pen_logout","pen_logout","pen_title",
                          "pen_align","pen_class") {
                $rps->{$arg[0]}{$pen} = 0;
            }
            Irpg::Irc::chanmsg("Welcome $usernick\'s new player $arg[0], the ".
                    "@arg[2..$#arg] Farmer! Next level in ".
                    duration($opts->{rpbase}).".");
            Irpg::Irc::privmsg("Success! Account $arg[0] created. You have ".
                    "$opts->{rpbase} seconds idleness until you ".
                    "reach level 1. ", $usernick);
            Irpg::Irc::privmsg("NOTE: The point of the game is to see who ".
                    "can idle the longest. As such, ".
                    "parting, quitting, and changing ".
                    "nicks all penalize you.",$usernick);
        }
    }
}


sub chtitle {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    my $title = join(' ', @arg);
    if (!defined($username)) {
        Irpg::Irc::notice("You are not logged in.", $usernick);
    }
    elsif (!$title) {
        Irpg::Irc::notice("Try: CHTITLE <new title>", $usernick);
    }
    else {
        $rps->{$username}{title} = $title;
        Irpg::Irc::privmsg("Your title was changed.",$source);
        Irpg::Irc::chanmsg("Title for $arg[0] changed to $title.");
        Irpg::Main::penalize($username,"chtitle");
    }
}


sub login {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    if (defined($username)) {
        Irpg::Irc::notice("Sorry, you are already online as $username.",
                $usernick);
    }
    else {
        if ($#arg < 1 || $arg[1] eq "") {
            Irpg::Irc::notice("Try: LOGIN <username> <password>", $usernick);
        }
        elsif (!exists $rps->{$arg[0]}) {
            Irpg::Irc::notice("Sorry, no such account name. Note that ".
                    "account names are case sensitive.",$usernick);
        }
        elsif (!exists $onchan->{$usernick}) {
            Irpg::Irc::notice("Sorry, you're not in $opts->{botchan}.",
                    $usernick);
        }
        elsif ($rps->{$arg[0]}{pass} ne
               crypt($arg[1],$rps->{$arg[0]}{pass})) {
            Irpg::Irc::notice("Wrong password.", $usernick);
        }
        else {
            if ($opts->{voiceonlogin}) {
            Irpg::Irc::sts("MODE $opts->{botchan} +v :$usernick");
            }
            $rps->{$arg[0]}{online} = 1;
            $rps->{$arg[0]}{nick} = $usernick;
            $rps->{$arg[0]}{userhost} = $userhost;
            $rps->{$arg[0]}{lastlogin} = time();
            Irpg::Irc::chanmsg("$arg[0], the level $rps->{$arg[0]}{level} ".
                    "$rps->{$arg[0]}{alignment} $rps->{$arg[0]}{title} ".
                    "$rps->{$arg[0]}{class}->{NAME}, ".
                    "is now online from nickname $usernick. Next level in ".
                    duration($rps->{$arg[0]}{next}).".");
            Irpg::Irc::notice("Logon successful. Next level in ".
                   duration($rps->{$arg[0]}{next}).".", $usernick);
        }
    }
}

sub logout {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    if (defined($username)) {
        $rps->{$username}{online}=0;
        Irpg::Main::penalize($username,"logout");
    }
    else {
        Irpg::Irc::privmsg("You are not logged in.", $usernick);
    }
}

sub status {
    return unless ($opts->{statuscmd});
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    my $asked = exists($arg[0]) ?
                    $arg[0] :
                    (defined($username) ?
                        $username : $usernick);

    $asked = Irpg::Main::finduser($asked) unless (exists($rps->{$asked}));
    $asked = Irpg::Main::finduser($asked, 1) unless ($asked);

    if (!$asked) {
        Irpg::Irc::privmsg("No matching user found.",$source);
    }
    else {
        Irpg::Irc::privmsg("$asked: Level $rps->{$asked}{level} ".
                "$rps->{$asked}{alignment} $rps->{$asked}{title} ".
                "$rps->{$asked}{class}->{NAME}; ".
                "Status: O".($rps->{$asked}{online}?"n":"ff")."line; ".
                "TTL: ".duration($rps->{$asked}{next})."; ".
                "Idled: ".duration($rps->{$asked}{idled})."; ".
                "Item sum: ".itemsum($asked).
                "; More details at ".$opts->{playerurl}.$asked
                ,$source);
    }
}

sub whoami {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    if (!defined($username)) {
        Irpg::Irc::notice("You are not logged in.", $usernick);
    }
    else {
        Irpg::Irc::privmsg("You are $username, the level ".
                "$rps->{$username}{level} $rps->{$username}{alignment} ".
                "$rps->{$username}{title} ".
                "$rps->{$username}{class}->{NAME}. You have ".
                "$rps->{$username}{points} point(s) available, and ".
                "$rps->{$username}{actions} action(s) you can trigger. ".
                "Next level in ".duration($rps->{$username}{next}),
                $source);
    }
}

sub newpass {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    if (!defined($username)) {
        Irpg::Irc::notice("You are not logged in.", $usernick);
    }
    elsif (!defined($arg[0])) {
        Irpg::Irc::privmsg("Try: NEWPASS <new password>", $usernick);
    }
    else {
        $rps->{$username}{pass} = crypt($arg[0],mksalt());
        Irpg::Irc::privmsg("Your password was changed.",$usernick);
    }
}

sub align {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    if (!defined($username)) {
        Irpg::Irc::notice("You are not logged in.", $usernick);
    }
    elsif (!defined($arg[0]) || (lc($arg[0]) ne "good" && 
           lc($arg[0]) ne "neutral" && lc($arg[0]) ne "evil")) {
        Irpg::Irc::notice("Try: ALIGN <good|neutral|evil>", $usernick);
    }
    else {
        $rps->{$username}{alignment} = substr(uc($arg[0]),0,1).
                                       substr(lc($arg[0]),1,length $arg[0]);
        Irpg::Irc::chanmsg("$username has changed alignment to: ".
                           "$rps->{$username}{alignment}.");
        Irpg::Irc::privmsg("Your alignment was changed to ".
                           "$rps->{$username}{alignment}.", $usernick);
        Irpg::Main::penalize($username,"align");
    }
}

sub gender {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    if (!defined($username)) {
        Irpg::Irc::notice("You are not logged in.", $usernick);
    }
    elsif (!defined($arg[0]) || $arg[0] =~ /[^MFNmfn]/) {
        Irpg::Irc::notice("Try: GENDER <F[emale]|N[eutral]|M[ale]>",$usernick);
    }
    else {
        $rps->{$username}{gender} = substr(lc($arg[0]),0,1);
        Irpg::Irc::chanmsg("$username has changed gender to: ".lc($arg[0]).
                ".");
        Irpg::Irc::privmsg("Your gender was changed to ".lc($arg[0]).".",
                $source);
    }
}

sub rmplayer {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    if (!defined($username)) {
        Irpg::Irc::notice("You are not logged in.", $usernick);
    }
    else {
        Irpg::Irc::privmsg("Account $username removed.",$usernick);
        Irpg::Irc::chanmsg("$usernick removed his account, $username, the ".
                $rps->{$username}{title}.".");
        delete($rps->{$username});
    }
}

sub info {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    my $info;
    my $admins = join(", ", map { $rps->{$_}{nick} }
                           grep { $rps->{$_}{isadmin} &&
                                  $rps->{$_}{online} }
                            keys(%$rps));
    if (!Irpg::Main::ha($usernick)) {
        $info = "IRPG bot by ElTata, ".
                "based on jotun's initial work.".
                "https://gennuso.iiens.net/irpg. On via: ".
                $opts->{servers}->[0].". Admins online: $admins";
        Irpg::Irc::privmsg($info, $usernick);
    }
    else {
        my $queuedbytes = 0;
        $queuedbytes += (length($_)+2) for @Irpg::Irc::queue; # +2 = \r\n

        $info = sprintf(
            "%.2fkb sent, %.2fkb received in %s. %d IRPG users ".
            "online of %d total users. PAUSE_MODE is %d, ".
            "SILENT_MODE is %d. Outgoing queue is %d bytes ".
            "in %d items. On via: %s. Admins online: %s.",
            $Irpg::Irc::outbytes/1024,
            $Irpg::Irc::inbytes/1024,
            duration(time()-$^T),
            scalar(grep { $rps->{$_}{online} } keys(%$rps)),
            scalar(keys(%$rps)),
            $$pausemode_ref,
            $$silentmode_ref,
            $queuedbytes,
            scalar(@Irpg::Irc::queue),
            $opts->{servers}->[0],
            $admins);
        Irpg::Irc::privmsg($info, $usernick, 1);
    }
}

sub points {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    if (!defined($username)) {
        Irpg::Irc::notice("You are not logged in.", $usernick);
        return;
    }
    if (!@arg) {
        Irpg::Irc::privmsg("Your raw competences are the following: ".
            join(', ', map { uc($_)." $rps->{$username}{stats}{$_}" }
                        qw(str con int wis cha dex)).". ".
            "You have $rps->{$username}{points} point(s) available.", $source);
        return;
    }
    if (@arg != 2 || $arg[1] !~ m/^\d+$/) {
        Irpg::Irc::notice("Try: POINTS [<stat> <n>]",$usernick);
        return;
    }
    my ($stat, $n) = @arg;
    $stat = lc($stat);
    if ($n > $rps->{$username}{points}) {
        Irpg::Irc::privmsg("You do not have that much points to spend.", $source);
        return;
    }
    if (!grep { $_ eq $stat } qw(str con wis int cha dex)) {
        Irpg::Irc::privmsg("'$stat' is not a valid stat name.", $source);
        return    
    }

    $rps->{$username}{stats}{$stat} += $n;
    $rps->{$username}{points} -= $n;
    Irpg::Irc::privmsg("You sure feel more competent now!", $usernick);
}

sub getclass {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    if (!defined($username)) {
        Irpg::Irc::notice("You are not logged in.", $usernick);
        return;
    }
    my $asked = exists($arg[0]) ? $arg[0] : $username;

    $asked = Irpg::Main::finduser($asked) unless (exists($rps->{$asked}));
    $asked = Irpg::Main::finduser($asked, 1) unless ($asked);

    if (!$asked) {
        Irpg::Irc::privmsg("No matching user found.",$source);
    }
    else {
        my $cname = $rps->{$asked}{class}->{NAME};
        my $who = $asked eq $username ? 'You are' : "$asked is";
        Irpg::Irc::privmsg("$who a".($cname =~ m/^[AEIOUYÆŒ].*/ ? 'n':'').
            " $cname with the following stats: ".
            join(', ', map { uc($_)." ".eval('$rps->{$asked}{class}->'.$_.'()') }
                        qw(str con int wis cha dex)).".", $source);
        return;
    }
}

sub chclass {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    my $cname = lc(join(' ', @arg));
    $cname =~ s/\b(\w)/\u$1/g;  # capitalize the first letter of each word
    my $cname_ = $cname;
    $cname_ =~ s/ /_/g;
    if (!defined($username)) {
        Irpg::Irc::notice("You are not logged in.", $usernick);
        return;
    }
    elsif (!$cname) {
        Irpg::Irc::notice("Try: CHCLASS <class name>", $usernick);
        return
    }
    my @classes;
    foreach (<Irpg/Classes/*.pm>) {
        s/Irpg\/Classes\/(\w+)\.pm/$1/;
        push @classes, $_;
    }
    if (!grep { $cname_ eq $_ } @classes) {
        Irpg::Irc::privmsg("I know you are a very special unique person ".
            "with your very own feelings, sensitivity, emotions, ".
            "competences and everything, but you really can't be ".
            "a".($cname =~ m/^[AEIOUYÆŒ].*/ ? 'n':'')." '$cname'. ".
            "Too bad :(.", $source);
    }
    elsif ($cname eq $rps->{$username}{class}->{NAME}) {
        Irpg::Irc::privmsg("You already ARE a".
            ($cname =~ m/^[AEIOUYÆŒ].*/ ? 'n':'')." '$cname', and ".
            "your are making a fool of yourself... This is really awkward.",
            $source)
    }
    else {
        eval "require Irpg::Classes::$cname_";
        my $new_class = eval 'Irpg::Classes::'.$cname_.'->new($rps->{$username}{stats})';
        if ($new_class) {
            $rps->{$username}{class} = $new_class;
            Irpg::Main::penalize($username, 'chclass');
            # show the new class
            getclass($userhost,$usernick,$username,$source);
            Irpg::Irc::chanmsg("$username has trained hard following the ".
                "demanding path ".pronoun(1, $rps->{$username}{gender}).
                " has chosen, and ".pronoun(2, $rps->{$username}{gender}).
                " efforts paid back as ".pronoun(1, $rps->{$username}{gender}).
                " today rises as a proud $cname.");
        }
        else {
            Irpg::Irc::privmsg("You tried the best you could, ".
                "but it seems you are lacking something... You do not ".
                "match the criteria to be part of the people going ".
                "by the name of ${cname}s, and as such, you are ".
                "kept away from their privileges.", $source);
        }
    }
}

our $commands = {
    register => {ref => \&register, adm => 0, prv => 1, pub => 0,
                 hlp => 'REGISTER <char name> <password <char title> :'.
                        'register a new player. <char name> <= 16 chars, '.
                        '<password> <= 8 chars, <char title> <= 30 chars.'},

    removeme => {ref => \&rmplayer, adm => 0, prv => 1, pub => 1,
                 hlp => 'REMOVEME : delete your account.'},

    login    => {ref => \&login, adm => 0, prv => 1, pub => 0,
                 hlp => 'LOGIN <char name> <password> : '.
                        'login as a character.'},

    logout   => {ref => \&logout, adm => 0, prv => 1, pub => 0,
                 hlp => 'LOGOUT <char name> <password> : '.
                        'logout from a character.'},

    newpass  => {ref => \&newpass, adm => 0, prv => 1, pub => 0,
                 hlp => 'NEWPASS <new password> : change password.'},

    chtitle  => {ref => \&chtitle, adm => 0, prv => 1, pub => 1,
                 hlp => 'CHTITLE <new title> : change title. '.
                        'Changing title is a p30 action.'},

    align    => {ref => \&align, adm => 0, prv => 1, pub => 1,
                 hlp => 'ALIGN <good|neutral|evil> : change alignment. '.
                        'Changing alignment is a p30 action.'},

    gender   => {ref => \&gender, adm => 0, prv => 1, pub => 1,
                 hlp => 'GENDER <F|N|M> : change your gender.'},

    status   => {ref => \&status,    adm => 0, prv => 1, pub => 1,
                 hlp => 'STATUS [<usernick>|<char name>] : '.
                        'get the status of a character, or of a '.
                        'usernick\'s character. Yours by default.'},

    whoami   => {ref => \&whoami, adm => 0, prv => 1, pub => 1,
                 hlp => 'WHOAMI : check your current status'},

    info     => {ref => \&info,        adm => 0, prv => 1, pub => 0,
                 hlp => 'INFO : get some info about the bot.'},

    points   => {ref => \&points, adm => 0, prv => 1, pub => 1,
                 hlp => 'POINTS [<stat> <n>] : shows your points repartition, '.
                         'or adds n points to your stat. '.
                         'stats are {str, con, wis, int, cha, dex}.'},

    class    => {ref => \&getclass, adm => 0, prv => 1, pub => 1,
                 hlp => 'CLASS [<usernick>|<char name>]: gives the class info '.
                        'of a character, or of a usernick\'s character. '.
                        'Yours by default.'},

    chclass  => {ref => \&chclass, adm => 0, prv => 1, pub => 1,
                 hlp => 'CHCLASS <class name>: apply for a new class membership. '.
                        'Changing class cost the TTL of your level.'}
};

1;
